#!/usr/bin/env python
import requests
from time import sleep
from datetime import datetime
from Adafruit_CharLCD import Adafruit_CharLCD
lcd = Adafruit_CharLCD(rs=26, en=19, d4=13, d5=6, d6=5, d7=11, cols=16, lines=2)
lcd.clear()

import Adafruit_DHT as dht

# Yahoo Weather City ID's more available at https://weather.yahoo.com
cities = [  ('Amsterdam', '727232'),
            ('Athens', '946738'),
            ('Bangkok',  '1225448'),
            ('Bariloche', '465562'),
            ('Beijing', '2151330'),
            ('Berlin', '638242'),
            ('Bern', '781788'),
            ('Bratislava', '818717'),
            ('Brussels', '968019'),
            ('Bucharest', '868274'),
            ('Budapest', '804365'),
            ('Cape Town', '1591691'),
            ('Chicago', '2379574'),
            ('Copenhagen', '554890'),
            ('Dhaka', '1915035'),
            ('Dublin', '560743'),
            ('Hanoi', '91888417'),
            ('Helsinki', '565346'),
            ('Istanbul', '2344116'),
            ('Jakarta', '1047378'),
            ('Jerusalem', '1968222'),
            ('Kiev', '924938'),
            ('Kuantan', '1154799'),
            ('Lima', '418440'),
            ('Lisbon', '742676'),
            ('Ljubljana', '530634'),
            ('London',   '44418'),
            ('Los Angeles', '12795609'),
            ('Madrid', '766273'),
            ('Manila', '1199477'),
            ('Melbourne', '1103816'),
            ('Mexico City', '116545'),
            ('Montreal', '3534'),
            ('Moscow', '2122265'),
            ('Mumbai', '2295411'),
            ('New York', '2459115'),
            ('Oslo', '862592'),
            ('Paris', '615702'),
            ('Prague', '796597'),
            ('Rio', '455825'),
            ('Rome', '721943'),
            ('Santiago', '349859'),
            ('Seoul', '1132599'),
            ('Singapore', '1062617'),
            ('Sofia', '839722'),
            ('Stockholm', '906057'),
            ('Taipei', '2306179'),
            ('Tokyo', '1118370'),
            ('Venice', '725746'),
            ('Vienna', '551801'),
            ('Warsaw', '523920')]

city_codes = ",".join([c[1] for c in cities])

def getWeatherConditions() :
    try :
        url = 'https://query.yahooapis.com/v1/public/yql'
        # encode query string for request
        query = ('select item.condition from weather.forecast where woeid in ('
                + city_codes + ')')
        query_strings = {'q': query, 'format' : 'json'}
        # headers to disable caching (in theory)
        headers = {'Pragma': 'no-cache', 'Cache-Control': 'no-cache'}
        # get weather data from Yahoo
        r = requests.get(url, params=query_strings, headers=headers)
        # return data formatted to JSON
        return r.json()
    except Exception:
        return "Feed Error"
    

if __name__ == "__main__":
    while 1:
        data = getWeatherConditions()
        if( type(data) != type(dict()) or "error" in data):
            # Error
            print (data)
            lcd.clear()
            lcd.message("FEED\nERROR")
            sleep(4)
        else:
            count = data["query"]["count"]
            # Loop through cities
            for c in range(0, count):
                # Retrieve city name, condition and temperature
                city = cities[c][0]
                cond = data["query"]["results"]["channel"][c]["item"]["condition"]["text"]
                temp = data["query"]["results"]["channel"][c]["item"]["condition"]["temp"]
                lcd.clear()
                # Display city temperature and condition
                lcd.message(city + " " + temp)
                lcd.write8(223, True)
                lcd.message("F\n" + cond)
                sleep(2)
                # Poll DHT22
                h,t = dht.read_retry(dht.DHT22, 4)
                t = t * 9/5.0 + 32
                lcd.clear()
                # Display time
                lcd.message(datetime.now().strftime('%I:%M %p %b %d'))
                # Display temperature an humidity
                lcd.message("\n{0:0.1f}".format(t))
                lcd.write8(223, True)
                lcd.message("F   {0:0.1f}%".format(h))
                
                sleep(2)
